/*
Map class
7 October 2005
*/

namespace tlib
{

/*
Keys must be unique
*/
template <class K, class V> class TSlowMap
{
public:
	TSlowMap()
	{
	}
	TSlowMap(const TSlowMap<K, V>& rhs)
	{
		FKeyList = rhs.FKeyList;
		FValueList = rhs.FValueList;
	}

	~TSlowMap()
	{
	}
	void Reset()
	{
		FKeyList.Clear();
		FValueList.Clear();
	}

	TSlowMap<K, V>& operator =(const TSlowMap<K, V>& rhs)
	{
		FKeyList = rhs.FKeyList;
		FValueList = rhs.FValueList;
		return *this;
	}

	void Add(K key, V value)
	{
		for (int i = 0; i < FKeyList.NumItems; i++)
		{
			if (FKeyList[i] == key)
				throw TException(L"Key already exists");
		}
		FKeyList.Add(key);
		FValueList.Add(value);
	}

	void Set(K key, V value)
	{
		for (int i = 0; i < FKeyList.NumItems; i++)
		{
			if (FKeyList[i] == key)
			{
				FValueList[i] = value;
				return;
			}
		}
		FKeyList.Add(key);
		FValueList.Add(value);
	}

	void Remove(K key)
	{
		for (int i = 0; i < FKeyList.NumItems; i++)
		{
			if (FKeyList[i] == key)
			{
				FKeyList.Remove(i);
				FValueList.Remove(i);
				return;
			}
		}
		throw TException(L"Key does not exist");
	}

	V& operator [](K key) const
	{
		for (int i = 0; i < FKeyList.NumItems; i++)
		{
			if (FKeyList[i] == key)
				return FValueList[i];
		}
		throw TException(L"Key does not exist");
	}

	bool ContainsKey(K key) const
	{
		for (int i = 0; i < FKeyList.NumItems; i++)
		{
			if (FKeyList[i] == key)
				return true;
		}
		return false;
	}

	void ReplaceKey(K oldKey, K key)
	{
		for (int i = 0; i < FKeyList.NumItems; i++)
		{
			if (FKeyList[i] == key)
				throw TException("New key already exists");
		}
		for (int i = 0; i < FKeyList.NumItems; i++)
		{
			if (FKeyList[i] == oldKey)
			{
				FKeyList[i] = key;
				return;
			}
		}
	}

	PROPERTY_C(KeyList, TArray<K>&, FKeyList)
	PROPERTY_C(ValueList, TArray<V>&, FValueList)

	PROPERTY_F(NumEntries, int)

private:
	TArray<K> FKeyList;
	TArray<V> FValueList;

	int _GetNumEntries() const
	{
		return FKeyList.NumItems;
	}
};

}   // namespace tlib